\name{shannonInformation}
\alias{laplaceProbability}
\alias{laplaceInformation}
\alias{shannonInformation}
\alias{absoluteEntropy}
\alias{normalizedEntropy}
\alias{redundancy}
\alias{distributionEntropy}
\alias{densityEntropy}
\title{Univariate Informationtheoretic Measures}
\description{
  These functions calculate univariate informationtheoretic measures, like \emph{information}, \emph{entropy} and \emph{redundancy}.
}
\usage{
laplaceProbability(n)
laplaceInformation(n)
shannonInformation(p)
absoluteEntropy(p)
normalizedEntropy(p)
redundancy(p)
distributionEntropy(x, FUN = pnorm, ..., normalized = TRUE)
densityEntropy(lower = -Inf, upper = Inf, FUN = dnorm, ..., normalized = TRUE)
}
\arguments{
  \item{n}{ scalar: number of (equal likely) possibilities }
  \item{p}{ vector of probabilities }
  \item{x}{ sequence of values at which FUN is evaluated }
  \item{lower}{ lower bound for integrate }
  \item{upper}{ upper bound for integrate }
  \item{FUN}{ a cumulative distribution function }
  \item{\dots}{ further arguments to FUN }
  \item{normalized}{ FALSE to use \code{absoluteEntropy} instead of TRUE (default) \code{normalizedEntropy} }
}
\value{
  Function \code{shannonInformation} returns a vector of log2 (or zero where p=0). Entropy functions return vector of entropy contributions for each p.
}
\references{ MacKay, David J.C. (2003). \emph{Information Theory, Inference, and Learning Algorithms (chapter 8).} Cambridge University Press.}
\author{ Jens Oehlschlaegel }
\seealso{ \code{\link{twowayEntropy}},  \code{\link{KullbackLeibler}}, \code{log}, \code{\link{exactMarginInfo}} }
\examples{

  # elementary
  laplaceInformation(2)      # = log2(n)
  p <- laplaceProbability(2) # = rep(1/n, n)
  p
  shannonInformation(p)      # = log2(p)
  absoluteEntropy(p)         # = p*log2(p)
  normalizedEntropy(p)       # = p*log2(p) / log2(n)

  sum(absoluteEntropy(p))    # Max = log2(n)
  sum(normalizedEntropy(p))  # Max = 1
  sum(redundancy(p))          # Redundancy = 1 - NormalizedEntropy

  laplaceInformation(2)
  laplaceInformation(4)  # more categories, more information
  sum(absoluteEntropy(laplaceProbability(2)))
  sum(absoluteEntropy(laplaceProbability(4)))   # more categories, more entropy
  sum(normalizedEntropy(laplaceProbability(2)))
  sum(normalizedEntropy(laplaceProbability(4))) # more categories, constant normalized entropy
  p <- c(0.5, 0.25, 0.15, 0.1)
  sum(normalizedEntropy(p))                      # unequal probabilities, lower entropy (normalized or not)

  sum(distributionEntropy(seq(-3, 3, 0.01)))
  sum(distributionEntropy(seq(-3, 3, 0.001)))
  sum(distributionEntropy(seq(-30, 30, 0.01), sd=10))
  sum(distributionEntropy(seq(-3, 3, 0.01), FUN=punif, -3, 3))

  sum(distributionEntropy(seq(-3, 3, 0.01), normalized=FALSE))
  sum(distributionEntropy(seq(-3, 3, 0.001), normalized=FALSE))
  sum(distributionEntropy(seq(-30, 30, 0.01), sd=10, normalized=FALSE))
  sum(distributionEntropy(seq(-3, 3, 0.01), FUN=punif, -3, 3, normalized=FALSE))

  densityEntropy(-3, 3)
  densityEntropy(-30, 30, sd=10)

}
\keyword{ univar }
